//  MenuBar.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//
// Gaspar Sinai <gsinai@gol.com>
//


#include <stdio.h>
#include <stdlib.h>
#include "MenuBar.h"
#include <Translate.h>


MenuBar::MenuBar (MenuBarAction* prnt, const Config*  configIn)  : QObject ()
{
	int			i;
	int			j;
	QString			str;
	AString  		strItem;
	AString			defaultItem;


	config = configIn;

	for (i=0; i<Config::CHECKLIST_MAX; i++)
	{
		configMenu[i] = 0;
		configMenuId[i] = -1;
	}
	fileMenu = 0;
	windowMenu = 0;

	//
	// File
	//
	fileMenu = new QPopupMenu();
	fileMenu->insertItem (Translate ("&Open..."), this, 
			SLOT (loadFileSlot()), ALT+Key_L);

	fileMenu->insertItem (Translate ("&Append..."), this, 
			SLOT (appendFileSlot()), ALT+Key_A);

	fileMenu->insertItem (Translate ("&Re-Load"), this, 
			SLOT (reloadFileSlot()), ALT+Key_R);

	fileMenu->insertItem (Translate ("&New Window"), this, 
			SLOT (newSlot()), ALT+Key_N);

	fileMenu->insertSeparator ();

	fileMenu->insertItem (Translate ("&Save"), this, 
			SLOT (saveFileSlot()), ALT+Key_S);

	fileMenu->insertItem (Translate ("&Save As..."), this, 
			SLOT (saveAsFileSlot()), ALT+Key_V);

	fileMenu->insertSeparator ();
	fileMenu->insertItem (Translate ("Confi&g"), this, 
			SLOT (configSlot()), ALT+Key_G);
	fileMenu->insertItem (Translate ("&Print"), this, 
			SLOT (printSlot()), ALT+Key_P);

	fileMenu->insertSeparator ();

	fileMenu->insertItem (Translate ("&Close"), this, 
			SLOT (quitSlot()), ALT+Key_C);

	fileMenu->insertItem (Translate ("&Exit All"), this, 
			SLOT (exitSlot()), ALT+Key_E);

	prnt->aInsert (Translate ("&File"), fileMenu);

	//
	// All popup menus
	//
	for (i=0; i<Config::CHECKLIST_MAX; i++)
	{
		configMenu[i] = new QPopupMenu ();
		configMenu[i]->setCheckable (True);
		configMenuId[i] = -1;
		defaultItem = (const char*) config->getChecked (
			(Config::CheckList)i);
		for (j=0; j<(config->getCheckList ((Config::CheckList)i)).getSize(); j++)
		{
			strItem = (const char*) 
			   (config->getCheckList ((Config::CheckList)i)).at(j);
			configMenu[i]->insertItem ((const char*) strItem);
			if (strcmp ((const char*) strItem, 
				(const char*) defaultItem) ==0)
			{
				configMenuId[i] = configMenu[i]->idAt (j);
				if (configMenuId[i]!=-1)
				{
					configMenu[i]->setItemChecked (
						configMenuId[i], True);
				}
			}
		}
		switch (i)
		{
		case Config::ENCODING:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (encodingSlot (int)));
			break;
		case Config::INPUT:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (inputSlot (int)));
			break;
		case Config::XINPUT:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (xInputSlot (int)));
			break;
		case Config::FONTNAME:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (fontNameSlot (int)));
			break;
		case Config::FONTWEIGHT:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (fontWeightSlot (int)));
			break;
		case Config::FONTSIZE:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (fontSizeSlot (int)));
			break;
		case Config::FONTSLANT:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (fontSlantSlot (int)));
			break;
		case Config::FONTSPACING:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (fontSpacingSlot (int)));
			break;
		case Config::FONT_ADD_STYLE:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (fontAddStyleSlot (int)));
			break;
		case Config::FONT_AVG_WIDTH:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (fontAvgWidthSlot (int)));
			break;
		case Config::TABSIZE:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (tabSizeSlot (int)));
			break;
		case Config::WINDOW_BACKGROUND:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (windowBackgroundSlot (int)));
			break;
		case Config::WINDOW_FOREGROUND:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (windowForegroundSlot (int)));
			break;
		case Config::CURSOR_COLOR:
			connect (configMenu[i], SIGNAL (activated (int)), 
				this, SLOT (windowCursorColorSlot (int)));
			break;
			
		}
	}
	prnt->aInsert (Translate ("&Encoding"), configMenu[Config::ENCODING]);
	prnt->aInsert (Translate ("&Input"), configMenu[Config::INPUT]);

	configMenu[Config::INPUT]->insertSeparator();
	configMenu[Config::INPUT]->insertItem (Translate ("&XInput"), 
		configMenu[Config::XINPUT]);

	configMenu[Config::FONTNAME]->insertSeparator ();
	configMenu[Config::FONTNAME]->insertItem (
		Translate ("&Size"), configMenu[Config::FONTSIZE]);
	configMenu[Config::FONTNAME]->insertItem (
		Translate ("&Weight"), configMenu[Config::FONTWEIGHT]);
	configMenu[Config::FONTNAME]->insertItem (
		Translate ("S&lant"), configMenu[Config::FONTSLANT]);
	configMenu[Config::FONTNAME]->insertItem (
		Translate ("S&pacing"), configMenu[Config::FONTSPACING]);
	configMenu[Config::FONTNAME]->insertItem (
		Translate ("&Add.Style"), configMenu[Config::FONT_ADD_STYLE]);
	configMenu[Config::FONTNAME]->insertItem (
		Translate ("A&vg.Width"), configMenu[Config::FONT_AVG_WIDTH]);

	prnt->aInsert (Translate ("&Font"), configMenu[Config::FONTNAME]);

	windowMenu = new QPopupMenu ();

	windowMenu->insertItem ( Translate ("&Tab Size"), 
		configMenu[Config::TABSIZE]);
	windowMenu->insertItem ( Translate ("&Background"), 
		configMenu[Config::WINDOW_BACKGROUND]);
	windowMenu->insertItem ( Translate ("&Foreground"), 
		configMenu[Config::WINDOW_FOREGROUND]);
	windowMenu->insertItem ( Translate ("&Cursor Color"), 
		configMenu[Config::CURSOR_COLOR]);

	prnt->aInsert (Translate ("&Window"), windowMenu);

	searchMenu = new QPopupMenu();
	searchMenu->insertItem (Translate ("&Find/Replace"), this,
			SLOT (searchFindSlot()), ALT+Key_F);
	searchMenu->insertItem (Translate ("&Go To"), this,
			SLOT (searchGotoSlot()), ALT+Key_F);
	prnt->aInsert (Translate ("&Search"), searchMenu);

	helpMenu = new QPopupMenu();
	helpMenu->insertItem (Translate ("&About"), this, 
			SLOT (helpAboutSlot()), ALT+Key_A);

	helpMenu->insertSeparator ();

	helpMenu->insertItem (Translate ("&General"), this, 
			SLOT (helpGeneralSlot()), ALT+Key_G);

	helpMenu->insertItem (Translate ("&Config"), this, 
			SLOT (helpConfigSlot()), ALT+Key_C);

	helpMenu->insertSeparator ();

	helpMenu->insertItem (Translate ("&TestPage"), this, 
			SLOT (helpTestPageSlot()), ALT+Key_T);

	prnt->aInsert (Translate ("&Help"), helpMenu);


	checkGeneric (Config::ENCODING);
}

MenuBar::~MenuBar ()
{
	int	i;

	delete fileMenu;
	delete windowMenu;
	delete helpMenu;
	
	for (i=0; i<Config::CHECKLIST_MAX; i++)
	{
		delete configMenu[i];
	}
}

void
MenuBar::checkGeneric (Config::CheckList list)
{
	int		i;

	const char* inputName = (const char*) config->getChecked (list);

	if (configMenuId[list]  != -1)
	{
		configMenu[list]->setItemChecked (configMenuId[list], False);
	}
	if (inputName==0)
	{
		configMenuId[list] = configMenu[list]->idAt (0);
		configMenu[list]->setItemChecked (configMenuId[list], True);
		return;
	}
	
	for (i=0;;i++)
	{
		configMenuId[list] = configMenu[list]->idAt (i);
		if (configMenuId[list] == -1) break;
		if (strcmp (configMenu[list]->text (configMenuId[list]), 
			inputName)==0)
		{
			configMenu[list]->setItemChecked (configMenuId[list],
				 True);
			break;
		}
	}
}

void
MenuBar::encodingSlot (int i)
{
	const char* text = configMenu[Config::ENCODING]->text(i);

	emit changeConfigSignal (Config::ENCODING, text);
	checkGeneric(Config::ENCODING);
}

void
MenuBar::inputSlot (int i)
{
	const char* text = configMenu[Config::INPUT]->text(i);
	emit changeConfigSignal (Config::INPUT, text);
	checkGeneric(Config::INPUT);
}

void
MenuBar::xInputSlot (int i)
{
	const char* text = configMenu[Config::XINPUT]->text(i);
	emit changeConfigSignal (Config::XINPUT, text);
	checkGeneric(Config::XINPUT);
}

void
MenuBar::fontNameSlot (int i)
{
	const char* text = configMenu[Config::FONTNAME]->text(i);

	emit changeConfigSignal (Config::FONTNAME, text);
	checkGeneric(Config::FONTNAME);
}


void
MenuBar::fontWeightSlot (int i)
{
	const char* text = configMenu[Config::FONTWEIGHT]->text(i);
	emit changeConfigSignal (Config::FONTWEIGHT, text);
	checkGeneric(Config::FONTWEIGHT);
}


void
MenuBar::fontSlantSlot (int i)
{
	const char* text = configMenu[Config::FONTSLANT]->text(i);

	emit changeConfigSignal (Config::FONTSLANT, text);
	checkGeneric(Config::FONTSLANT);
}

void
MenuBar::fontSpacingSlot (int i)
{
	const char* text = configMenu[Config::FONTSPACING]->text(i);

	emit changeConfigSignal (Config::FONTSPACING, text);
	checkGeneric(Config::FONTSPACING);
}

void
MenuBar::fontAddStyleSlot (int i)
{
	const char* text = configMenu[Config::FONT_ADD_STYLE]->text(i);

	emit changeConfigSignal (Config::FONT_ADD_STYLE, text);
	checkGeneric(Config::FONT_ADD_STYLE);
}

void
MenuBar::fontAvgWidthSlot (int i)
{
	const char* text = configMenu[Config::FONT_AVG_WIDTH]->text(i);

	emit changeConfigSignal (Config::FONT_AVG_WIDTH, text);
	checkGeneric(Config::FONT_AVG_WIDTH);
}

void
MenuBar::fontSizeSlot (int i)
{
	const char* text = configMenu[Config::FONTSIZE]->text(i);

	emit changeConfigSignal (Config::FONTSIZE, text);
	checkGeneric(Config::FONTSIZE);
}

void
MenuBar::windowForegroundSlot (int i)
{
	const char* text = configMenu[Config::WINDOW_FOREGROUND]->text(i);

	emit changeConfigSignal (Config::WINDOW_FOREGROUND, text);
	checkGeneric(Config::WINDOW_FOREGROUND);
}

void
MenuBar::windowCursorColorSlot (int i)
{
	const char* text = configMenu[Config::CURSOR_COLOR]->text(i);

	emit changeConfigSignal (Config::CURSOR_COLOR, text);
	checkGeneric(Config::CURSOR_COLOR);
}

void
MenuBar::tabSizeSlot (int i)
{
	const char* text = configMenu[Config::TABSIZE]->text(i);

	emit changeConfigSignal (Config::TABSIZE, text);
	checkGeneric(Config::TABSIZE);
}

void
MenuBar::windowBackgroundSlot (int i)
{
	const char* text = configMenu[Config::WINDOW_BACKGROUND]->text(i);

	emit changeConfigSignal (Config::WINDOW_BACKGROUND, text);
	checkGeneric(Config::WINDOW_BACKGROUND);
}

void
MenuBar::newSlot ()
{
	emit newSignal ();
}

void
MenuBar::configSlot ()
{
	emit configSignal ();
}

void
MenuBar::printSlot ()
{
	emit printSignal ();
}

void
MenuBar::quitSlot ()
{
	emit quitSignal ();
}

void
MenuBar::exitSlot ()
{
	emit exitSignal ();
}

void
MenuBar::saveAsFileSlot ()
{
	emit saveAsFile ();
}

void
MenuBar::saveFileSlot ()
{
	emit saveFile ();
}

void
MenuBar::loadFileSlot ()
{
	emit loadFileSignal ();
}

void
MenuBar::appendFileSlot ()
{
	emit appendFileSignal ();
}

void
MenuBar::reloadFileSlot ()
{
	emit reloadFileSignal ();
}

void
MenuBar::searchFindSlot ()
{
	emit searchSignal (FIND);
}

void
MenuBar::searchGotoSlot ()
{
	emit searchSignal (GOTO);
}

void
MenuBar::helpAboutSlot ()
{
	emit helpSignal (ABOUT);
}

void
MenuBar::helpGeneralSlot ()
{
	emit helpSignal (GENERAL);
}

void
MenuBar::helpConfigSlot ()
{
	emit helpSignal (CONFIG);
}

void
MenuBar::helpTestPageSlot ()
{
	emit helpSignal (TESTPAGE);
}


int
MenuBarAction::aInsert (const char* text, QPopupMenu* menu)
{
	return 0;
}
